"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isIsolatedElement = isIsolatedElement;
var isolatedElements = {};

var getIsolatedElements = exports.getIsolatedElements = function getIsolatedElements() {
  return isolatedElements;
};
var resetIsolatedElements = exports.resetIsolatedElements = function resetIsolatedElements() {
  isolatedElements = {};
};

function isIsolatedElement(elm) {
  var keys = Object.keys(isolatedElements);
  for (var i = 0; i < keys.length; ++i) {
    if (elm === isolatedElements[keys[i]]) {
      return keys[i].trim();
    }
  }
  return false;
}

function setScope(elm, scope) {
  isolatedElements[scope] = elm;
}

function removeScope(scope) {
  delete isolatedElements[scope];
}

function update(oldVNode, vNode) {
  var _oldVNode$data = oldVNode.data;
  var oldData = _oldVNode$data === undefined ? {} : _oldVNode$data;
  var elm = vNode.elm;
  var _vNode$data = vNode.data;
  var data = _vNode$data === undefined ? {} : _vNode$data;


  var oldIsolate = oldData.isolate || "";
  var isolate = data.isolate || "";

  if (isolate) {
    removeScope(oldIsolate);
    setScope(elm, isolate);
  }
  if (oldIsolate && !isolate) {
    removeScope(isolate);
  }
}

function remove(_ref, cb) {
  var _ref$data = _ref.data;
  var data = _ref$data === undefined ? {} : _ref$data;

  if (data.isolate) {
    removeScope(data.isolate);
  }
  cb();
}

function destroy(_ref2) {
  var _ref2$data = _ref2.data;
  var data = _ref2$data === undefined ? {} : _ref2$data;

  if (data.isolate) {
    removeScope(data.isolate);
  }
}

var IsolateModule = {
  create: update,
  update: update,
  remove: remove,
  destroy: destroy
};

exports.IsolateModule = IsolateModule;