'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.makeIsStrictlyInRootScope = exports.makeElementSelector = undefined;

var _utils = require('./utils');

var _events = require('./events');

var _isolate = require('./isolate');

var _makeIsStrictlyInRootScope = require('./makeIsStrictlyInRootScope');

var _isolate2 = require('./modules/isolate');

var matchesSelector = void 0;
try {
  matchesSelector = require('matches-selector');
} catch (e) {
  matchesSelector = function matchesSelector() {};
}

function sortIsolatedNamespace(a) {
  return a.indexOf(_utils.SCOPE_PREFIX) !== -1 ? 1 : -1;
}

function makeFindElements(namespace) {
  return function findElements(rootElement) {
    if (namespace.join('') === '') {
      return rootElement;
    }
    var slice = Array.prototype.slice;

    var scope = (0, _utils.getScope)(namespace);
    var selector = (0, _utils.getSelectors)(namespace);
    var topNode = rootElement;
    var topNodeMatches = [];

    if (scope.length > 0) {
      topNode = (0, _isolate2.getIsolatedElements)()[scope];
      if (matchesSelector(topNode, selector)) {
        topNodeMatches.push(topNode);
      }
    }

    return slice.call(topNode.querySelectorAll(selector)).filter((0, _makeIsStrictlyInRootScope.makeIsStrictlyInRootScope)(scope)).concat(topNodeMatches);
  };
}

function makeElementSelector(rootElement$, namespace) {
  return function elementSelector(selector) {
    if (typeof selector !== 'string') {
      throw new Error('DOM driver\'s select() expects the argument to be a ' + 'string as a CSS selector');
    }

    var trimmedSelector = selector.trim();
    var childNamespace = trimmedSelector === ':root' ? namespace : namespace.concat(trimmedSelector).sort(sortIsolatedNamespace);

    return {
      observable: rootElement$.map(makeFindElements(childNamespace)),
      namespace: childNamespace,
      select: makeElementSelector(rootElement$, childNamespace),
      events: (0, _events.makeEventsSelector)(rootElement$, childNamespace),
      isolateSource: _isolate.isolateSource,
      isolateSink: _isolate.isolateSink
    };
  };
}

exports.makeElementSelector = makeElementSelector;
exports.makeIsStrictlyInRootScope = _makeIsStrictlyInRootScope.makeIsStrictlyInRootScope;